{% load i18n %}

<div class="detail">
  <dl class="dl-horizontal">

    <dt>{% trans "ID" %}</dt>
    <dd>{{ execution.id }}</dd>

    <dt>{% trans "Function ID" %}</dt>
    <dd>{{ execution.function_id }}</dd>

    <dt>{% trans "Function Version" %}</dt>
    <dd>{{ execution.function_version }}</dd>

    <dt>{% trans "Description" %}</dt>
    <dd>{{ execution.description }}</dd>

    <dt>{% trans "Input" %}</dt>
    <dd>{{ execution.input }}</dd>

    <dt>{% trans "Result" %}</dt>
    <dd>
      <table class="table">
        <thead>
          <tr>
            <th>{% trans "Duration" %}</th>
            <th>{% trans "Output" %}</th>
          </tr>
        </thead>
        <tbody>
          <tr>
            <td>{{ result.duration }}</td>
            <td>{{ result.output }}</td>
          </tr>
        </tbody>
      </table>
    </dd>

    <dt>{% trans "Sync" %}</dt>
    <dd>{{ execution.sync }}</dd>

    <dt>{% trans "Project ID" %}</dt>
    <dd>{{ execution.project_id }}</dd>

    <dt>{% trans "Created At" %}</dt>
    <dd>{{ execution.created_at|parse_isotime }}</dd>

    <dt>{% trans "Updated At" %}</dt>
    <dd>{{ execution.updated_at|parse_isotime }}</dd>

    <dt>{% trans "Status" %}</dt>
    {% with execution.status|title as rt_status %}
    <dd>{% trans rt_status context "current status of a runtime" %}</dd>
    {% endwith %}

  </dl>
</div>
